﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using NDream.AirConsole;
using Newtonsoft.Json.Linq;

public class AirController : MonoBehaviour {
    [SerializeField]PlayerCharacter playerPrefab;
    public Dictionary<int, PlayerCharacter> playerDictionary = new Dictionary<int, PlayerCharacter>();

    const int PLAYER_MOVE = 0;
    const int PLAYER_JUMP = 1;
    int playersReady = 0;
    int teamOneNumPlayers = 0;
    int teamTwoNumPlayers = 0;
	bool gameStarted = false;
	[SerializeField] GameObject startPanel;


    public Transform[] teamOStarts;
    public Transform[] teamIStarts;
    public Color[] teamColors;



    void OnEnable()
    {
        AirConsole.instance.onMessage += OnMessage;
        AirConsole.instance.onConnect += OnDeviceConnect;
        AirConsole.instance.onDisconnect += OnDeviceDisconnect;
    }

    void OnDisable()
    {
        AirConsole.instance.onMessage -= OnMessage;
        AirConsole.instance.onConnect -= OnDeviceConnect;
        AirConsole.instance.onDisconnect -= OnDeviceDisconnect;
    }


    void OnMessage(int deviceId, JToken message)
    {
        if (gameStarted && message["joystick-left"] != null)
        {
            Vector2 _input = Vector2.zero;
            if ((bool)message["joystick-left"]["pressed"])
            {
                _input = new Vector2((float)message["joystick-left"]["message"]["x"], (float)message["joystick-left"]["message"]["y"]);
            }
            MessagePlayerMove(deviceId, _input);
        } else if (message["jump"] != null)
        {
            if (gameStarted)
            {
                MessagePlayerJump(deviceId);
            } else
            {
                ToggleReady(deviceId);
            }
        }
    }
    void OnDeviceConnect(int deviceId)
    {
        PlayerCharacter _player = Instantiate(playerPrefab, Vector3.up, Quaternion.identity) as PlayerCharacter;
        _player.teamId = playerDictionary.Count % 2;
        _player.SetColor(teamColors[_player.teamId]);
        int offset = Mathf.FloorToInt(playerDictionary.Count-1 / 2);
        if (_player.teamId == 0)
        {
            Vector3 _pos = teamOStarts[teamOneNumPlayers].position;
            teamOneNumPlayers++;
            _player.SetSpawn(_pos);
        } else
        {
            Vector3 _pos = teamIStarts[teamTwoNumPlayers].position;
            teamTwoNumPlayers++;
            _player.SetSpawn(_pos);
        }
        playerDictionary.Add(deviceId, _player);
        _player.ResetPosition();
        _player.SetName(AirConsole.instance.GetNickname(deviceId));
    }
    void OnDeviceDisconnect(int deviceId)
    {
        GameObject _player = playerDictionary[deviceId].gameObject;
        playerDictionary.Remove(deviceId);
        Destroy(_player);
    }

    void MessagePlayerMove(int id, Vector2 move)
    {
        playerDictionary[id].Move(move);
    }
    void MessagePlayerJump(int id)
    {
        playerDictionary[id].Jump();
    }

    void ToggleReady(int id)
    {
        playerDictionary[id].ready = !playerDictionary[id].ready;
        playersReady += (playerDictionary[id].ready) ? 1 : -1;
		if (playersReady >= 1){
			gameStarted = true;
			startPanel.SetActive (false);
		} 
    }
}
