﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;

[RequireComponent(typeof(Rigidbody))]
public class PlayerCharacter : MonoBehaviour {

    [SerializeField]
    Transform shadow;
    const float minDistanceToJump = 0.666f; //Hail Satan;
    bool allowJump = false;
    const float skinwitdh = 0.001f;
    [SerializeField]LayerMask ground;
    Vector2 playerInput;
    [SerializeField]
    float jumpSpeed = 400f;
    [SerializeField]
    float stompSpeed = 1000f;
    Rigidbody rb;
    [SerializeField]float moveSpeed = 5f;
    [SerializeField]
    ShockWave shockwavePrefab;
    float shockHeight = -1f;
    Collider ownCollision;
    public bool ready = false;
	public int teamId;

    SpriteRenderer spriteRenderer;
    Animator anim;
	Vector3 spawn;
    [SerializeField]
    Text playerName;
    
    [SerializeField]
    AudioClip[] audioClips;

	void Start () {
        rb = GetComponent<Rigidbody>();
        ownCollision = GetComponent<Collider>();
        anim = GetComponentInChildren<Animator>();
        spriteRenderer = GetComponentInChildren<SpriteRenderer>();

    }

    public void SetColor(Color _col)
    {
        if (spriteRenderer == null)
            spriteRenderer = GetComponentInChildren<SpriteRenderer>();
        spriteRenderer.color = _col;
    }

    public void SetSpawn(Vector3 _pos)
    {
        spawn = _pos;
    }

    public void SetName(string _name)
    {
        playerName.text = _name;
    }

    public void ResetPosition()
    {
        transform.position = spawn;
        if (rb == null)
        rb = GetComponent<Rigidbody>();
        rb.velocity = Vector3.zero;
    }

    void Update()
    {
        SetShadow();
        if (transform.position.y < -10f) ResetPosition();
    }

    void SetShadow()
    {
        Ray _r = new Ray(transform.position, Vector3.down);
        RaycastHit _hit;
        Physics.Raycast(_r, out _hit,ground);
        float _d = 2f;
        float _distance = 99f;
        if (_hit.collider != null)
        {
            _distance = transform.position.y - _hit.point.y;
            shadow.position = _hit.point + Vector3.up * skinwitdh;
            _d = _d / (transform.position.y - _hit.point.y);

        }
        bool _tmp = allowJump;
        allowJump = (_distance < minDistanceToJump);
        if (_tmp != allowJump)
        {
            if (shockHeight > 0)
            {
                SendShockwave(shockHeight);
                shockHeight = -1f;
            }
            anim.SetBool("ground", allowJump);
        }
        shadow.localScale = new Vector3(1*_d, 1*_d, 1);
    }

	void FixedUpdate () {
        Vector3 _dpos = new Vector3(playerInput.x,0f,-playerInput.y)* moveSpeed * Time.fixedDeltaTime;
        rb.MovePosition(transform.position + _dpos);
	}

    public void Move(Vector2 move)
    {
        if (allowJump)
        playerInput = move;
        if (move.x > 0.01f)
        {
            spriteRenderer.flipX = false;
        } else if (move.x < -0.01f)
        {
            spriteRenderer.flipX = true;
        }
            anim.SetFloat("move", Mathf.Abs(move.x) + Mathf.Abs(move.y));
    }
    public void Jump()
    {
        if (!allowJump) JumpEnd();
        else JumpStart();
    }
    void JumpStart()
    {
        anim.SetBool("ground", false);
        Vector3 _force = (new Vector3(playerInput.x, 0f, -playerInput.y) + Vector3.up).normalized;
        playerInput = Vector2.zero;
        rb.AddForce(_force * jumpSpeed);
        anim.SetTrigger("jump");
        //AudioSource.PlayClipAtPoint(audioClips[1], transform.position);
        AudioSource.PlayClipAtPoint(audioClips[1], Camera.main.transform.position + Camera.main.transform.forward, 0.35f);
    }
    void JumpEnd()
    {
        anim.SetTrigger("stomp");
        rb.AddForce(Vector3.down * stompSpeed);
        shockHeight = transform.position.y - shadow.position.y;
    }
    void SendShockwave(float _str)
    {
        AudioSource.PlayClipAtPoint(audioClips[0], Camera.main.transform.position + Camera.main.transform.forward, 0.35f);
        ShockWave _s = Instantiate(shockwavePrefab, shadow.position, Quaternion.identity) as ShockWave;
        _s.Init(_str * 2f, ownCollision);
    }
}
